CREATE TABLE universes (
	id TEXT PRIMARY KEY NOT NULL,
	name TEXT NOT NULL,

	UNIQUE (name)
);

CREATE TABLE projects (
	id TEXT PRIMARY KEY NOT NULL,
	universe_id TEXT NOT NULL,
	name TEXT NOT NULL,

	UNIQUE (universe_id, name),

	FOREIGN KEY (universe_id)
		REFERENCES universes (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION
);

CREATE TABLE metrics (
	id TEXT PRIMARY KEY NOT NULL,
	project_id TEXT NOT NULL,
	metrics_set_id TEXT NOT NULL,
	name TEXT NOT NULL,
	query TEXT NOT NULL,

	UNIQUE (project_id, name),

	FOREIGN KEY (metrics_set_id)
		REFERENCES metrics_sets (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION,

	FOREIGN KEY (project_id)
		REFERENCES projects (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION
);

CREATE TABLE metrics_sets (
	id TEXT PRIMARY KEY NOT NULL,
	project_id TEXT NOT NULL,
	name TEXT NOT NULL,

	UNIQUE (project_id, name),

	FOREIGN KEY (project_id)
		REFERENCES projects (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION
);

CREATE TABLE metrics_stores (
	id TEXT PRIMARY KEY NOT NULL,
	project_id TEXT NOT NULL,
	adapter TEXT NOT NULL,
	direction TEXT CHECK(direction IN ('push', 'pull')) NOT NULL,
	configuration TEXT NOT NULL,
	last_tx BIGINT DEFAULT 0 NOT NULL,
	last_ts DOUBLE PRECISION NOT NULL,
	period DOUBLE PRECISION DEFAULT 60.0 NOT NULL,

	FOREIGN KEY (project_id)
		REFERENCES projects (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION
);

CREATE TABLE metrics_exports (
	id TEXT PRIMARY KEY NOT NULL,
	project_id TEXT NOT NULL,
	metrics_store_id TEXT NOT NULL,
	metrics_set_id TEXT NOT NULL,

	UNIQUE (metrics_store_id, metrics_set_id),

	FOREIGN KEY (metrics_store_id)
		REFERENCES metrics_stores (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION,

	FOREIGN KEY (metrics_set_id)
		REFERENCES metrics_sets (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION,

	FOREIGN KEY (project_id)
		REFERENCES projects (id)
		ON DELETE CASCADE
		ON UPDATE NO ACTION
);
